@@ -1,5 +1,5 @@
 /* Return sibling of given DIE.
-   Copyright (C) 2003, 2004, 2005 Red Hat, Inc.
+   Copyright (C) 2003, 2004, 2005, 2007 Red Hat, Inc.
    This file is part of Red Hat elfutils.
    Written by Ulrich Drepper <drepper@redhat.com>, 2003.
 
@@ -92,7 +92,7 @@ dwarf_siblingof (die, result)
 	{
 	  Dwarf_Off offset;
 	  sibattr.valp = addr;
-	  if (INTUSE(dwarf_formref) (&sibattr, &offset) != 0)
+	  if (__libdw_formref (&sibattr, &offset) != 0)
 	    /* Something went wrong.  */
 	    return -1;
 
