@@ -148,12 +148,15 @@ __elfw2(LIBELFBITS,updatenull) (Elf *elf, int *change_bop, size_t shnum)
 
   /* Set the program header position.  */
   if (elf->state.ELFW(elf,LIBELFBITS).phdr == NULL
-      && (ehdr->e_type == ET_EXEC || ehdr->e_type == ET_DYN))
+      && (ehdr->e_type == ET_EXEC || ehdr->e_type == ET_DYN
+	  || ehdr->e_type == ET_CORE))
     (void) INTUSE(elfw2(LIBELFBITS,getphdr)) (elf);
   if (elf->state.ELFW(elf,LIBELFBITS).phdr != NULL)
     {
-      /* Only executables or shared objects have a program header.  */
-      if (ehdr->e_type != ET_EXEC && unlikely (ehdr->e_type != ET_DYN))
+      /* Only executables, shared objects, and core files have a program
+	 header.  */
+      if (ehdr->e_type != ET_EXEC && ehdr->e_type != ET_DYN
+	  && unlikely (ehdr->e_type != ET_CORE))
 	{
 	  __libelf_seterrno (ELF_E_INVALID_PHDR);
 	  return -1;
