@@ -1,15 +1,27 @@
-/* Copyright (C) 2001, 2002, 2003, 2004, 2005 Red Hat, Inc.
+/* Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Red Hat, Inc.
+   This file is part of Red Hat elfutils.
    Written by Ulrich Drepper <drepper@redhat.com>, 2001.
 
-   This program is Open Source software; you can redistribute it and/or
-   modify it under the terms of the Open Software License version 1.0 as
-   published by the Open Source Initiative.
+   Red Hat elfutils is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by the
+   Free Software Foundation; version 2 of the License.
 
-   You should have received a copy of the Open Software License along
-   with this program; if not, you may obtain a copy of the Open Software
-   License version 1.0 from http://www.opensource.org/licenses/osl.php or
-   by writing the Open Source Initiative c/o Lawrence Rosen, Esq.,
-   3001 King Ranch Road, Ukiah, CA 95482.   */
+   Red Hat elfutils is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public License along
+   with Red Hat elfutils; if not, write to the Free Software Foundation,
+   Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301 USA.
+
+   Red Hat elfutils is an included package of the Open Invention Network.
+   An included package of the Open Invention Network is a package for which
+   Open Invention Network licensees cross-license their patents.  No patent
+   license is granted, either expressly or impliedly, by designation as an
+   included package.  Should you wish to participate in the Open Invention
+   Network licensing program, please visit www.openinventionnetwork.com
+   <http://www.openinventionnetwork.com>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -60,7 +72,7 @@ elf_i386_relocate_section (struct ld_state *statep __attribute__ ((unused)),
   Elf_Data *data;
 
   /* Iterate over all the input sections.  Appropriate data buffers in the
-     output sections were already created.  I get them iteratively, too.  */
+     output sections were already created.  */
   runp = firstp;
   data = NULL;
   do
@@ -137,8 +149,8 @@ elf_i386_relocate_section (struct ld_state *statep __attribute__ ((unused)),
 	  assert (xndx < SHN_LORESERVE || xndx > SHN_HIRESERVE);
 
 	  /* We fortunately don't have to do much.  The relocations
-	     mostly get only updates of the offset.  Only is a
-	     relocation referred to a section do we have to do
+	     mostly get only updates of the offset.  Only for a
+	     relocation referring to a section do we have to do
 	     something.  In this case the reference to the sections
 	     has no direct equivalent since the part the input section
 	     contributes need not start at the same offset as in the
@@ -147,15 +159,13 @@ elf_i386_relocate_section (struct ld_state *statep __attribute__ ((unused)),
 	     itself.  */
 	  if (XELF_ST_TYPE (sym->st_info) == STT_SECTION)
 	    {
-	      Elf32_Word toadd;
-
-	      /* We expect here on R_386_32 relocations.  */
+	      /* We expect here only R_386_32 relocations.  */
 	      assert (XELF_R_TYPE (rel->r_info) == R_386_32);
 
 	      /* Avoid writing to the section memory if this is
 		 effectively a no-op since it might save a
 		 copy-on-write operation.  */
-	      toadd = file->scninfo[xndx].offset;
+	      Elf32_Word toadd = file->scninfo[xndx].offset;
 	      if (toadd != 0)
 		add_4ubyte_unaligned (reltgtdata->d_buf + rel->r_offset,
 				      toadd);
@@ -384,14 +394,19 @@ elf_i386_finalize_plt (struct ld_state *statep, size_t nsym,
       /* Point the GOT entry at the PLT entry, after the initial jmp.  */
       ((Elf32_Word *) data->d_buf)[3 + cnt] = pltentryaddr + 6;
 
-      /* The value of the symbol is the address of the corresponding PLT
-	 entry.  Store the address, also for the normal symbol table if
-	 this is necessary.  */
-      ((Elf32_Sym *) dynsymdata->d_buf)[1 + cnt].st_value = pltentryaddr;
 
-      if (symdata != NULL)
-	((Elf32_Sym *) symdata->d_buf)[nsym - statep->nplt + cnt].st_value
-	  = pltentryaddr;
+      /* If the symbol is defined, adjust the address.  */
+      if (((Elf32_Sym *) dynsymdata->d_buf)[1 + cnt].st_shndx != SHN_UNDEF)
+	{
+	  /* The value of the symbol is the address of the corresponding PLT
+	     entry.  Store the address, also for the normal symbol table if
+	     this is necessary.  */
+	  ((Elf32_Sym *) dynsymdata->d_buf)[1 + cnt].st_value = pltentryaddr;
+
+	  if (symdata != NULL)
+	    ((Elf32_Sym *) symdata->d_buf)[nsym - statep->nplt + cnt].st_value
+	      = pltentryaddr;
+	}
     }
 
   /* Create the .plt section.  */
@@ -508,10 +523,17 @@ elf_i386_count_relocations (struct ld_state *statep, struct scninfo *scninfo)
 	{
 	  Elf32_Word r_sym = XELF_R_SYM (rel->r_info);
 
+	  /* Symbols in COMDAT group sections which are discarded do
+	     not have to be relocated.  */
+	  if (r_sym >= scninfo->fileinfo->nlocalsymbols
+	      && unlikely (scninfo->fileinfo->symref[r_sym] == NULL))
+	    continue;
+
 	  switch (XELF_R_TYPE (rel->r_info))
 	    {
 	    case R_386_GOT32:
-	      if (! scninfo->fileinfo->symref[r_sym]->defined)
+	      if (! scninfo->fileinfo->symref[r_sym]->defined
+		  || scninfo->fileinfo->symref[r_sym]->in_dso)
 		relsize += sizeof (Elf32_Rel);
 
 	      /* This relocation is not emitted in the output file but
@@ -534,6 +556,8 @@ elf_i386_count_relocations (struct ld_state *statep, struct scninfo *scninfo)
 		  if (statep->file_type == dso_file_type)
 		    {
 		      relsize += sizeof (Elf32_Rel);
+		      // XXX Do we have to check whether the target
+		      // XXX section is read-only first?
 		      statep->dt_flags |= DF_TEXTREL;
 		    }
 		  else
@@ -553,10 +577,9 @@ elf_i386_count_relocations (struct ld_state *statep, struct scninfo *scninfo)
 		    }
 		}
 	      else if (statep->file_type == dso_file_type
-		       && r_sym >= SCNINFO_SHDR (scninfo->fileinfo->scninfo[shdr->sh_link].shdr).sh_info
-		       && scninfo->fileinfo->symref[r_sym]->outdynsymidx != 0
 		       && XELF_R_TYPE (rel->r_info) == R_386_32)
 		relsize += sizeof (Elf32_Rel);
+
 	      break;
 
 	    case R_386_PLT32:
@@ -680,8 +703,7 @@ elf_i386_create_relocations (struct ld_state *statep,
       /* Cache the access to the symbol table data.  */
       Elf_Data *symdata = elf_getdata (scninfo[rshdr->sh_link].scn, NULL);
 
-      int cnt;
-      for (cnt = 0; cnt < nrels; ++cnt)
+      for (int cnt = 0; cnt < nrels; ++cnt)
 	{
 	  XElf_Rel_vardef (rel);
 	  XElf_Rel *rel2;
@@ -700,16 +722,24 @@ elf_i386_create_relocations (struct ld_state *statep,
 		 section in the output file and the addend.  */
 	      value = scninfo[sym->st_shndx].offset + sym->st_value;
 	    }
-	  else if (symref[idx]->in_dso)
-	    {
-	      /* MERGE.VALUE contains the PLT index.  We have to add 1 since
-		 there is this one special PLT entry at the beginning.  */
-	      assert (symref[idx]->merge.value != 0
-		      || symref[idx]->type != STT_FUNC);
-	      value = pltaddr + symref[idx]->merge.value * PLT_ENTRY_SIZE;
-	    }
 	  else
-	    value = symref[idx]->merge.value;
+	    {
+	      if (symref[idx] == NULL)
+		/* Symbol in ignored COMDAT group section.  */
+		continue;
+
+	      if (symref[idx]->in_dso)
+		{
+		  /* MERGE.VALUE contains the PLT index.  We have to
+		     add 1 since there is this one special PLT entry
+		     at the beginning.  */
+		  assert (symref[idx]->merge.value != 0
+			  || symref[idx]->type != STT_FUNC);
+		  value = pltaddr + symref[idx]->merge.value * PLT_ENTRY_SIZE;
+		}
+	      else
+		value = symref[idx]->merge.value;
+	    }
 
 	  /* Address of the relocated memory in the data buffer.  */
 	  void *relloc = (char *) data->d_buf + rel->r_offset;
@@ -782,8 +812,7 @@ elf_i386_create_relocations (struct ld_state *statep,
 		    }
 		}
 	      else if (statep->file_type == dso_file_type
-		       && idx >= SCNINFO_SHDR (scninfo[rshdr->sh_link].shdr).sh_info
-		       && symref[idx]->outdynsymidx != 0)
+		       && XELF_R_TYPE (rel->r_info) == R_386_32)
 		{
 #if NATIVE_ELF != 0
 		  xelf_getrel_ptr (reldyndata, nreldyn, rel2);
@@ -791,8 +820,15 @@ elf_i386_create_relocations (struct ld_state *statep,
 		  rel2 = &rel_mem;
 #endif
 		  rel2->r_offset = value;
-		  rel2->r_info
-		    = XELF_R_INFO (symref[idx]->outdynsymidx, R_386_32);
+
+		  /* For symbols we do not export we generate a relative
+		     relocation.  */
+		  if (idx < SCNINFO_SHDR (scninfo[rshdr->sh_link].shdr).sh_info
+		      || symref[idx]->outdynsymidx == 0)
+		    rel2->r_info = XELF_R_INFO (0, R_386_RELATIVE);
+		  else
+		    rel2->r_info
+		      = XELF_R_INFO (symref[idx]->outdynsymidx, R_386_32);
 		  (void) xelf_update_rel (reldyndata, nreldyn, rel2);
 		  ++nreldyn;
 
