@@ -134,7 +134,7 @@ static int handle_elf (Elf *elf, const char *prefix, const char *fname,
 
 #define INTERNAL_ERROR(fname) \
   error (EXIT_FAILURE, 0, gettext ("%s: INTERNAL ERROR %d (%s-%s): %s"),      \
-	 fname, __LINE__, VERSION, __DATE__, elf_errmsg (-1))
+	 fname, __LINE__, PACKAGE_VERSION, __DATE__, elf_errmsg (-1))
 
 
 /* Internal representation of symbols.  */
@@ -217,10 +217,10 @@ main (int argc, char *argv[])
   (void) setlocale (LC_ALL, "");
 
   /* Make sure the message catalog can be found.  */
-  (void) bindtextdomain (PACKAGE, LOCALEDIR);
+  (void) bindtextdomain (PACKAGE_TARNAME, LOCALEDIR);
 
   /* Initialize the message catalog.  */
-  (void) textdomain (PACKAGE);
+  (void) textdomain (PACKAGE_TARNAME);
 
   /* Parse and process arguments.  */
   (void) argp_parse (&argp, argc, argv, 0, &remaining, NULL);
@@ -249,7 +249,7 @@ main (int argc, char *argv[])
 static void
 print_version (FILE *stream, struct argp_state *state __attribute__ ((unused)))
 {
-  fprintf (stream, "nm (%s) %s\n", PACKAGE_NAME, VERSION);
+  fprintf (stream, "nm (%s) %s\n", PACKAGE_NAME, PACKAGE_VERSION);
   fprintf (stream, gettext ("\
 Copyright (C) %s Red Hat, Inc.\n\
 This is free software; see the source for copying conditions.  There is NO\n\
@@ -938,34 +938,39 @@ show_symbols_posix (Elf *elf, GElf_Word strndx, const char *prefix,
 /* Maximum size of memory we allocate on the stack.  */
 #define MAX_STACK_ALLOC	65536
 
+static int
+sort_by_address (const void *p1, const void *p2)
+{
+  GElf_SymX *s1 = (GElf_SymX *) p1;
+  GElf_SymX *s2 = (GElf_SymX *) p2;
+
+  int result = (s1->sym.st_value < s2->sym.st_value
+		? -1 : (s1->sym.st_value == s2->sym.st_value ? 0 : 1));
+
+  return reverse_sort ? -result : result;
+}
+
+static Elf_Data *sort_by_name_strtab;
+
+static int
+sort_by_name (const void *p1, const void *p2)
+{
+  GElf_SymX *s1 = (GElf_SymX *) p1;
+  GElf_SymX *s2 = (GElf_SymX *) p2;
+
+  const char *n1 = sort_by_name_strtab->d_buf + s1->sym.st_name;
+  const char *n2 = sort_by_name_strtab->d_buf + s2->sym.st_name;
+
+  int result = strcmp (n1, n2);
+
+  return reverse_sort ? -result : result;
+}
+
 static void
 show_symbols (Ebl *ebl, GElf_Ehdr *ehdr, Elf_Scn *scn, Elf_Scn *xndxscn,
 	      GElf_Shdr *shdr, const char *prefix, const char *fname,
 	      const char *fullname)
 {
-  int sort_by_name (const void *p1, const void *p2)
-    {
-      GElf_SymX *s1 = (GElf_SymX *) p1;
-      GElf_SymX *s2 = (GElf_SymX *) p2;
-      int result;
-
-      result = strcmp (elf_strptr (ebl->elf, shdr->sh_link, s1->sym.st_name),
-		       elf_strptr (ebl->elf, shdr->sh_link, s2->sym.st_name));
-
-      return reverse_sort ? -result : result;
-    }
-
-  int sort_by_address (const void *p1, const void *p2)
-    {
-      GElf_SymX *s1 = (GElf_SymX *) p1;
-      GElf_SymX *s2 = (GElf_SymX *) p2;
-
-      int result = (s1->sym.st_value < s2->sym.st_value
-		    ? -1 : (s1->sym.st_value == s2->sym.st_value ? 0 : 1));
-
-      return reverse_sort ? -result : result;
-    }
-
   /* Get the section header string table index.  */
   size_t shstrndx;
   if (elf_getshstrndx (ebl->elf, &shstrndx) < 0)
@@ -1142,7 +1147,11 @@ show_symbols (Ebl *ebl, GElf_Ehdr *ehdr, Elf_Scn *scn, Elf_Scn *xndxscn,
 
   /* Sort the entries according to the users wishes.  */
   if (sort == sort_name)
-    qsort (sym_mem, nentries, sizeof (GElf_SymX), sort_by_name);
+    {
+      sort_by_name_strtab = elf_getdata (elf_getscn (ebl->elf, shdr->sh_link),
+					 NULL);
+      qsort (sym_mem, nentries, sizeof (GElf_SymX), sort_by_name);
+    }
   else if (sort == sort_numeric)
     qsort (sym_mem, nentries, sizeof (GElf_SymX), sort_by_address);
 
